unit GrChars;
{$I Common.inc}
//************************************************************************
//      ,          *
//                                                                       *
//  :                                                     *
// TNodeChar -- -                                           *
// TGraphChars  -- ,   -                  *
//************************************************************************

interface

uses Root, Graph;

type

  // TNodeChar -- -
  TNodeChar = class (TNode)
    private
      function GetLinkByName(aName : char): TLink;
    public
      mName  : char;  //  
      constructor Create(aName: char; aVal: integer; aOwner: TGraph);
      function  Copy: TItem; override;
      function  GetName: string;  override;
  end;

  // TGraphChars  -- ,   -
  TGraphChars = class (TGraph)
    private
      function  GetLinkByName(aSource, aDest : char): TLink;
      procedure MakeLink(aSource, aDest : char; aVal : integer);
    public
      constructor Load(const aName: String);
      constructor GenRandom(aDir: boolean;
                            aLoadNodes, aLoadLinks,
                            aNodes, aLinks : integer);
      constructor GenFull(aDir: boolean;
                          aLoadNodes, aLoadLinks, aNodes : integer);
      procedure LoadFlowData(const aName: String);
      procedure Save(const aName: String); override;
      function  GetNode(aName : char): TNodeChar;
  end;

//   ,    

function GenCharsGraph(aDir: boolean;          // 
                       aLoadNodes, aLoadLinks, // 
                       aNodes,                 //  
                       aLinks                  //  ,%
                       : integer): TGraph;

//////////////////////////////////////////////////////////////////////////
implementation

uses SysUtils;
//////////////////////////////////////////////////////////////////////////

//   ,    

function GenCharsGraph(aDir: boolean;          // 
                       aLoadNodes, aLoadLinks, // 
                       aNodes,                 //  
                       aLinks                  //  ,%
                       : integer): TGraph;
begin
  if aLinks >= 100 {%}
    then Result:= TGraphChars.GenFull(aDir,         // 
                            aLoadNodes, aLoadLinks, // 
                            aNodes                  //  
                            )
    else Result:= TGraphChars.GenRandom(aDir,       // 
                            aLoadNodes, aLoadLinks, // 
                            aNodes,                 //  
                            aLinks                  //  ,%
                            );
end;

//////////////////////////////////////////////////////////////////////////
// TNodeChar -- -
//////////////////////////////////////////////////////////////////////////

constructor TNodeChar.Create(aName: char; aVal: integer; aOwner: TGraph);
begin
  inherited Create(aVal, aOwner);
  mName:= aName;
end;


function TNodeChar.Copy: TItem;
begin
  Result:= TNodeChar.Create(mName, mValue, mOwner);
end;

//      

function TNodeChar.GetLinkByName(aName: char): TLink;
begin
  OutPosPush;
  Result:= OutLinkFirst;
  while Assigned(Result) do begin
    if (Result.mDest as TNodeChar).mName = aName then Break;
    Result:= OutLinkNext;
  end;
  OutPosPop;
end;

function TNodeChar.GetName: string;
begin
  Result:= mName;  //   -  
end;

//////////////////////////////////////////////////////////////////////////
// TGraphChars --   -
//////////////////////////////////////////////////////////////////////////

//   

constructor TGraphChars.GenFull(aDir: boolean;
                                aLoadNodes, aLoadLinks, aNodes: integer);
const CArr : array[boolean] of char = ('F','T');
var i : integer;
    N1, N2 : TNode;
    name: Char;
    nm : string;
begin
  //   :
  nm:='Full: '+ CArr[aDir] +' : '+
         IntToStr(aLoadNodes)+ ' : '+
         IntToStr(aLoadLinks)+ ' : '+
         IntToStr(aNodes);

  //   :
  inherited Create (nm, aDir, aLoadNodes > 0, aLoadLinks > 0);

  //  
  for i:= 0 to aNodes-1 do begin
    name:= Char(Ord('A')+i);
    InsertNode(TNodeChar.Create(name, 1+Random(aLoadNodes), Self));
  end;

  //      
  N1:= NodeFirst;
  while Assigned(N1) do begin
    PosPush;
    N2:= NodeFirst;
    while Assigned(N2) do begin
      if N1 <> N2 then SetLink(N1, N2, 1+Random(aLoadLinks));
      N2:= NodeNext;
    end;
    PosPop;
    N1:= NodeNext;
  end;
end;

//   
//  aNodes < 0,    ,   
//  aLinks < 0,    (%) ,   

constructor TGraphChars.GenRandom(
                            aDir: boolean;  //  
                            aLoadNodes,     //  
                            aLoadLinks,     //  
                            aNodes,         //   
                            aLinks:         //   , %
                            integer);

const CArr : array[boolean] of char = ('F','T');

var Nodes, Links : integer;
    i, Limit : integer;
    Link: TLink;
    name, nS, nD: Char;
    nm : string;

begin
  if aNodes > 0
    then  // -  
      Nodes:= 2 + aNodes div 2 + Random(aNodes div 2)
    else  // -  
      Nodes:= Abs(aNodes);
  Limit:= Nodes*(Nodes-1) div 2;  //   
  // - 
  if aLinks > 0 then begin
    //   
    Links:= aLinks;
    if Links > 100 then Links:= 100;
    Links:= Limit div 4 + Random((aLinks*Limit) div 100);
  end else begin
    //   
    Links:= Abs(aLinks);
    if Links > 100 then Links:= 100;
    Links:= (Links*Limit) div 100;
  end;

  //   :
  nm:='Random: '+ CArr[aDir] +' : '+
         IntToStr(aLoadNodes)+ ' : '+
         IntToStr(aLoadLinks)+ ' : '+
         IntToStr(Nodes)+' : '+
         IntToStr(Links);

  //   :
  inherited Create (nm, aDir, aLoadNodes > 0, aLoadLinks > 0);

  //  
  for i:= 0 to Nodes-1 do begin
    name:= Char(Ord('A')+i);
    InsertNode(TNodeChar.Create(name, 1+Random(aLoadNodes), Self));
  end;

  //  
  repeat
    nS:= Char(Ord('A')+Random(Nodes));
    nD:= Char(Ord('A')+Random(Nodes));
    if nS = nD then Continue;
    Link:= GetLinkByName(nS, nD);
    if not Assigned(Link) then begin
      MakeLink(nS, nD, 1+Random(aLoadLinks));
      Dec(Links);
    end;
  until Links <= 0;
end;

{                  
Gragh - 2
0 - (0), (1)
1 -  
1 -   ()
3 -  
A=1 B=3 C=2
A -> B=5 C=7
B -> A=5 C=6
C -> A=7 B=6
}

constructor TGraphChars.Load(const aName: String);
const CharsSet = ['a'..'z', 'A'..'Z'];
var F: TextFile;
    nodes, n, val : integer;
    nm : string;       //  
    name : char;       //   = 'A'..'Z'
    Dir, LN, LL : boolean;
    S, D : char;       //     
    node : TNodeChar;  //  
    err : boolean;     //  
begin
  if not SysUtils.FileExists(aName) then begin
    Fail;
    Exit;
  end;
  err:= false; n:= 0;
  AssignFile(F, aName); Reset(F);

  if not Eof(F) then Readln(F, nm) else err:= true;
  if not Eof(F) then Readln(F, n) else err:= true;
  Dir:= n<>0;   //  
  if not Eof(F) then Readln(F, n) else err:= true;
  LN:= n<>0;    //   
  if not Eof(F) then Readln(F, n) else err:= true;
  LL:= n<>0;    //   

  inherited Create(nm, Dir, LN, LL);  //   

  //  :
  if not Eof(F) then Readln(F, n) else err:= true;
  nodes:= n;

  //  
  while not err and not Eof(F) and (n>0) do begin
    name:= '0'; val:= 1;
    Dec(n);
    //   (a..z, A..Z)
    while not Eoln(F) do begin
      Read(F, name);
      if name in CharsSet then Break;
    end;
    //   
    if mLoadNodes then begin
      Read(F, S);     //   =
      Read(F, val);   //  
    end;
    node:= TNodeChar.Create(name, val, Self);
    if not InsertNode(node)
      then node.Free;  //   ,  
  end;
  Readln(F);

  if n>0 then err:= true;  //    

  //  
  while not err and not Eof(F) do begin
    val:= 1;
    S:='0'; D:='0';
    //  -
    while not Eoln(F) do begin
      Read(F, S);
      if S in CharsSet then Break;
    end;
    //  -
    while not Eoln(F) do begin
      //  
      while not Eoln(F) do begin
        Read(F, name);
        D:= name;
        if D in CharsSet then Break;
      end;
      if not (D in CharsSet) then Break;
      //   :
      if mLoadLinks then begin
         Read(F, name);   //   =
         Read(F, val);    //  
      end;
      MakeLink(S, D, val);
    end;  // while
    Readln(F);
    Dec(nodes);
    if nodes=0 then Break;  //   
  end;  // while
  Close(F);

  if err then begin
    if Assigned(mNodes) then begin
      mNodes.ClrAndDestroy;
      mNodes.Free;
    end;
    Fail;
  end;
end;

//      
//   :
{
FLOW: --  
XY = mL mH  -- ,  ,  
...
}
procedure TGraphChars.LoadFlowData(const aName: String);
var F: TextFile;
    S: string;
    C1, C2: char;
    Low, High : integer;
    L: TLink;
begin
  AssignFile(F, aName); Reset(F);
  //   
  while not Eof(F) do begin
    Readln(F,S);
    if Pos('FLOW',S)<>0 then Break;
  end;
  while not Eof(F) do begin
    // C1, C2 --     
    C1:='-'; C2:='-';
    if not Eoln(F) then Read(F, C1) else begin Readln(F); Continue end;
    if not Eoln(F) then Read(F, C2) else begin Readln(F); Continue end;
    L:= GetLinkByName(C1, C2);
    if Assigned(L) then begin
      while not Eoln(F) do begin
        Read(F,C1);
        if C1='=' then Break;
      end;
      if not Eoln(F) then Read(F, Low) else begin Readln(F); Continue end;
      if not Eoln(F) then Read(F, High) else begin Readln(F); Continue end;
      //    
      L.mLow:= Low;
      L.mHigh:= High;
    end;
    Readln(F);
  end;
  Close(F);
end;

//     

procedure TGraphChars.Save(const aName: String);
var F: TextFile;
    i, j: integer;
    N: TNodeChar;
    L: TLink;

begin
  AssignFile(F, aName);
  if FileExists(aName) then Append(F) else Rewrite(F);
  Writeln(F, mName);
  Writeln(F, Ord(mDirect), ' -   (1 = )');
  Writeln(F, Ord(mLoadNodes), ' -  (1 = )');
  Writeln(F, Ord(mLoadLinks), ' -  (1 = )');
  Writeln(F, mNodes.GetCount, ' -  ');

  for i:= 1 to mNodes.GetCount do begin
    N:= mNodes.GetItem(i) as TNodeChar;
    if mLoadNodes
      then Write(F, N.mName+'=', N.mValue, ' ')
      else Write(F, N.mName, ' ');
  end;
  Writeln(F);

  for i:= 1 to mNodes.GetCount do begin
    N:= TNodeChar(mNodes.GetItem(i));
    Write(F, N.mName,' -> ');
    for j:= 1 to N.OutGetCnt do begin
      L:= N.OutGetLink(j);
      if N.mOwner.mLoadLinks
        then Write(F, (L.mDest as TNodeChar).mName, '=', L.mValue:2,' ')
        else Write(F, (L.mDest as TNodeChar).mName,' ');
    end;
    Writeln(F);
  end;
  Close(F);
end;

//          

function TGraphChars.GetLinkByName(aSource, aDest: char): TLink;
var S: TNodeChar;
begin
  Result:= nil;
  S:= GetNode(aSource);
  if not Assigned(S) then Exit;
  Result:= S.GetLinkByName(aDest);
end;

//       

function TGraphChars.GetNode(aName: char): TNodeChar;
begin
  mNodes.PositionPush;
  Result:= TNodeChar(mNodes.GetFirst);
  while Assigned(Result) do begin
    if Result.mName = aName then Break;
    Result:= mNodes.GetNext as TNodeChar;
  end;
  mNodes.PositionPop;
end;

//        

procedure TGraphChars.MakeLink(aSource, aDest : char; aVal: integer);
var S, D: TNode;
begin
  if GetLinkByName(aSource, aDest) <> nil then Exit;
  S:= GetNode(aSource);
  D:= GetNode(aDest);
  if Assigned(S) and Assigned(D)
    then SetLink(S, D, aVal)
end;

//////////////////////////////////////////////////////////////////////////

end.
